#include "stlexam.h"
#pragma hdrstop
/**************************************************************************
 *
 * autoptr.cpp - Illustrate the use of the auto_ptr class
 *    Section 13
 *
 * $Id: autoptr.cpp,v 1.11 1996/08/28 01:18:57 smithey Exp $
 *
 ***************************************************************************
 *
 * (c) Copyright 1994, 1995 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#ifndef _RWSTD_HEADER_REQUIRES_HPP
#include <vector>
#include <utility>
#include <string>
#else
#include <vector.hpp>
#include <utility.hpp>
#include <string.hpp>
#endif

#ifdef _RW_STD_IOSTREAM
#include <iostream>
#else
#include <iostream.h>
#endif
     
// In this program we'll use auto_ptr to store pointers to strings
// in a vector without having to worry about deleting the strings
// themselves

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif        

template <class X>
ostream& operator<<(ostream& os, const auto_ptr<X>& a)
{
   if (a.get())
      cout << *a;
   return os;
}

#ifndef _RWSTD_NO_NAMESPACE
  }
#endif        

int main()
{
  #ifndef _RWSTD_NO_NAMESPACE
  using namespace std; 
  #endif        
  {
    // First the wrong way
    vector<string*,allocator<string*> > v;
    v.insert(v.begin(), new string("Florence"));
    v.insert(v.begin(), new string("Milan"));
    v.insert(v.begin(), new string("Venice"));
    
    // Now remove the first element
    v.erase(v.begin());
    // Whoops, memory leak
    // string("Venice") was removed, but not deleted
    // We were supposed to handle that ourselves
  }
  {
    // Now the right way
    vector<auto_ptr<string>,allocator<string>  > v;
    v.insert(v.begin(), auto_ptr<string>(new string("Florence")));
    v.insert(v.begin(), auto_ptr<string>(new string("Milan")));
    v.insert(v.begin(), auto_ptr<string>(new string("Venice")));
    // Everything is fine since auto_ptrs transfer ownership of their pointers when copied


    cout << endl << "The Vector Container of Strings Contains : " << endl << endl << "\t";

    //Now copy this to the output
    copy (v.begin(), v.end(), ostream_iterator<auto_ptr<string> ,char,char_traits<char> >(cout, "\n\t"));

    // Now remove the first element
    v.erase(v.begin());
   	
    // Success
    // When auto_ptr(string("Venice")) is erased (and destroyed) 
    // string("Venice") is deleted
	

    //Now show access the auto_ptr with -> operator 
    v[0]->assign("Rome");
	 
    //Copy the contents of the auto_ptr to the output
    cout << endl << "The first string in the vector container is now  " << v[0] << endl;
	 
    
  }

  return 0;
}

