#ifndef __LOCALE_VECTOR
#define __LOCALE_VECTOR
#pragma option push -b -a4 -Vx- -Ve- -w-inl -w-aus -w-sig

/***************************************************************************
 *
 * locvector - Declarations for locales local vector class
 *
 ***************************************************************************
 *
 * (c) Copyright 1994, 1995 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#ifndef _RWSTD_NO_NAMESPACE
namespace __rwstd {
using namespace std;
#endif


// Minute vector class
template <class T>
class locale_vector
{
public:
    typedef size_t size_type;
    typedef ptrdiff_t difference_type;
    typedef T value_type;
    typedef T& reference;
    typedef T* pointer;
    typedef T* iterator;

private:
  iterator first_;
  size_type size_;

public:
  locale_vector() : first_(0), size_(0) {;}
  locale_vector(size_type n) : first_(0), size_(n) 
  {
    if (n)
      first_ = new T[n];
  }
  locale_vector(size_type n, const T& v) : first_(0), size_(n) 
  {
    if (n)
      first_ = new T[n];
    while (n)
      first_[--n] = v;
  }
  locale_vector(const locale_vector<T>& lv) : first_(0), size_(0)
  {
    size_type n = size_ = lv.size();
    if (n)
      first_ = new T[n];
    while (n--)
      first_[n] = lv[n];
  }    
  ~locale_vector() { if (first_) delete [] first_;}
  const locale_vector& operator= (const locale_vector& lv) 
  {
    size_type n = size_ = lv.size();
    if (first_)
      delete [] first_;
    if (n)
      first_ = new T[n];
    while (n--)
      first_[n] = lv[n];
    return *this;
  }
  iterator begin() const { return iterator(first_); }
  iterator end() const { return iterator(first_+size_); }    
  size_type size() const { return size_; }

  T& operator[] (size_t i) { return first_[i]; }
  const T& operator[] (size_t i) const { return first_[i]; }

  // Resize array and copy old contents to new buffer. 
  iterator resize(size_t s)
  { 
     return resize(s,T()); 
  }
  iterator resize(size_t s, T v)
  {
    T *new_buf = new T[s];
    iterator j,k;
    size_type d = size_ < s ? size_ : s;

    for (j = first_, k = new_buf; j != first_+d; j++,k++)
      *k =  *j;

    while (d < s)
      new_buf[d++] = v;

    if (first_)
      delete [] first_;    
    first_ = new_buf;
    size_ = s;
    return first_;
  }
};

#ifndef _RWSTD_NO_NAMESPACE
} // namespace __rwstd
#endif

#pragma option pop
#endif // __LOCALE_VECTOR
