/* This file is included specially and does not have a normal header guard */
#pragma option push -b -a4 -Vx- -Ve- -w-inl -w-aus -w-sig
#ifndef __STD_STDEXCEPT

/***************************************************************************
 *
 * stdexcept - declarations for the Standard Library standard exception class
 *
 * $Id: stdexcept,v 1.41 1996/08/28 01:31:01 smithey Exp $
 *
 ***************************************************************************
 *
 * (c) Copyright 1994, 1995 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#include <stdcomp.h>

#ifndef _RWSTD_HEADER_REQUIRES_HPP
#include <string>
#else
#include <string.hpp>
#endif

#ifndef __RWSTD_EXCEPT_SEEN
#ifndef _RWSTD_HEADER_REQUIRES_HPP
#include <exception>
#else
#include <exception.hpp>
#endif

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif

// MSVC provides its own exception class and logic_error class.
// In order to allow the use of MSVC classes derived from these
// We have to use them to, instead of our own.  The drawback is
// that we don't have a logic_error(const string&) constructor any
// more.  Now we have a logic_error(const char *) constructor.
#ifndef _RWSTD_LOGIC_ERROR_DEFINED
class _RWSTDExport logic_error : public exception
{
  public:
    logic_error (const string& what_arg)  _RWSTD_THROW_SPEC_NULL
      : str_(what_arg)
    { ; }

    virtual ~logic_error ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }
#else
    ;
#endif 

    virtual const char * what () const  _RWSTD_THROW_SPEC_NULL
    {
        return str_.data();
    }

  private:
    string str_;
};
#endif

class _RWSTDExport domain_error : public logic_error
{
  public:
    domain_error (const string& what_arg) _RWSTD_THROW_SPEC_NULL 
#ifndef _RWSTD_LOGIC_ERROR_DEFINED
      : logic_error(what_arg) {;}
#else
      : logic_error(what_arg.c_str()) {;}
#endif

    virtual ~domain_error ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }  
#else
    ;
#endif 
};

class _RWSTDExport invalid_argument : public logic_error
{
  public:
    invalid_argument (const string& what_arg) _RWSTD_THROW_SPEC_NULL 
#ifndef  _RWSTD_LOGIC_ERROR_DEFINED
      : logic_error(what_arg) {;}
#else
      : logic_error(what_arg.c_str()) {;}
#endif

    virtual ~invalid_argument ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }  
#else
    ;
#endif
};

class _RWSTDExport length_error : public logic_error
{
  public:
    length_error (const string& what_arg) _RWSTD_THROW_SPEC_NULL 
#ifndef  _RWSTD_LOGIC_ERROR_DEFINED
      : logic_error(what_arg) {;}
#else
      : logic_error(what_arg.c_str()) {;}
#endif

    virtual ~length_error ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }
#else
    ;
#endif
};

class _RWSTDExport out_of_range : public logic_error
{
  public:
    out_of_range (const string& what_arg) _RWSTD_THROW_SPEC_NULL 
#ifndef  _RWSTD_LOGIC_ERROR_DEFINED
      : logic_error(what_arg) {;}
#else
      : logic_error(what_arg.c_str()) {;}
#endif

    virtual ~out_of_range ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }  
#else
    ;
#endif
};

class _RWSTDExport runtime_error : public exception
{
  public:
    runtime_error (const string& what_arg) _RWSTD_THROW_SPEC_NULL  
      : str_(what_arg)
    { ; }

    virtual ~runtime_error ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }  
#else
    ;
#endif

    virtual const char * what () const  _RWSTD_THROW_SPEC_NULL
    {
        return str_.data();
    }

  private:
    string str_;
};

class _RWSTDExport range_error : public runtime_error
{
  public:
    range_error (const string& what_arg) _RWSTD_THROW_SPEC_NULL 
      : runtime_error(what_arg) {;}

    virtual ~range_error ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }  
#else
    ;
#endif
};

class _RWSTDExport overflow_error : public runtime_error
{
  public:
    overflow_error (const string& what_arg) _RWSTD_THROW_SPEC_NULL 
      : runtime_error(what_arg) {;}

    virtual ~overflow_error ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }  
#else
    ;
#endif
};

class _RWSTDExport underflow_error : public runtime_error
{
  public:
    underflow_error (const string& what_arg) _RWSTD_THROW_SPEC_NULL 
      : runtime_error(what_arg) {;}

    virtual ~underflow_error ()  _RWSTD_THROW_SPEC_NULL
#ifndef HPPA_WA
    { ; }  
#else
    ;
#endif
};

#define __RWSTD_EXCEPT_SEEN

#ifndef _RWSTD_NO_NAMESPACE 
} 
#endif

#endif /*__RWSTD_EXCEPT_SEEN*/

//
// Yes, the complete file has been processed!
//
#define __STD_STDEXCEPT

#endif /*__STD_STDEXCEPT*/

#pragma option pop
