#ifndef __RW_TRAITS
#define __RW_TRAITS
#pragma option push -b -a4 -Vx- -Ve- -w-inl -w-aus -w-sig

/***************************************************************************
 *
 * traits - Declarations for char_traits 
 *
 *
 ***************************************************************************
 *
 * (c) Copyright 1994, 1995 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 **************************************************************************/

#include <string.h>
#include <stdio.h>

//
// Temporarily turn off the warnings under the Borland compiler that
// say 'Functions containing ... cannot be inlined'
//
#if defined(__BORLANDC__)
#pragma option -w-inl
#endif

/*
 * this are all necessary for the "traits" class
 */
#ifdef _RW_STD_IOSTREAM
#include <rw/iotraits>
#endif

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif


typedef long         INT_T;


/*
 *     STRUCT CHAR_TRAITS
 */

template<class charT>
struct _RWSTDExportTemplate char_traits
{

        typedef charT                     char_type;
        typedef INT_T                     int_type;

#ifdef _RW_STD_IOSTREAM
        typedef mbstate_t                 state_type;
        typedef fpos<state_type>         pos_type;
        typedef wstreamoff               off_type;
#endif /* _RW_STD_IOSTREAM */

        static void assign (char_type& c1, const char_type& c2)   
        { c1 = c2;}

  	static char_type to_char_type(const int_type& c)
  	{ return c; }

	static int_type to_int_type(const char_type& c)
  	{ return c; }

	static bool     eq(const char_type& c1,const char_type& c2)
  	{ return (c1 == c2); }

        static bool lt (const char_type& c1, const char_type& c2) 
        { return c1 < c2;}

        static int compare (const char_type* s1, const char_type* s2, size_t n)
        {
          size_t i=0;

          while ( i < n )
           { 
             if ( !eq( s1[i], s2[i] ) )
              {
                 if ( lt( s1[i], s2[i]) )
                  return -1;
                 else
                  return 1;               
               }
             i++;
           }
         return 0;
        }
	
  	static bool     eq_int_type(const int_type& c1,const int_type& c2)
  	{ return (c1 == c2); }

  	static int_type             eof()
  	{ return EOF; }

  	static int_type             not_eof(const int_type& c)
  	{
    		if(c == EOF)
      		   return 0;
    		else
      		   return c;
  	}

  	static size_t               length(const char_type *s)
  	{
    	  size_t l = 0;
          while ( !eq(*s++, char_type(0) ) )  ++l;
          return l;
  	}
 
        static const char_type* 
        find (const char_type* s, int n, const char_type& a)
        {
          while (n-- > 0 && *s != a) 
            ++s;
          return  n >= 0 ? s : 0;
        }

  	static char_type    *copy(char_type *dst,const char_type *src, size_t n)
  	{
          return (char_type*) memcpy((char *)dst,(char *)src,
                                     n*sizeof(char_type));
  	}

        static char_type* move (char_type* s1, const char_type* s2, size_t n)
        {
          char_type * s = s1;
          if (s1 < s2)
           copy(s1, s2, n);
          else if (s1 > s2)
          {
           s1 += n;
           s2 += n;
           for(size_t i = 0; i < n; ++i) assign(*--s1, *--s2);
          }
          return s;
         }

        static char_type* 
        assign (char_type* s, size_t n, const char_type& a)
        {
          char_type* tmp = s;
          while (n-- > 0) 
            *tmp++ = a;
          return s;
        }

  };


/*
 *
 *     STRUCT CHAR_TRAITS SPECIALIZED FOR CHAR
 *
 */

_RWSTD_TEMPLATE
struct _RWSTDExport char_traits<char> 
{

  	typedef char                      char_type;
  	typedef int                       int_type;
       
#ifdef _RW_STD_IOSTREAM
        typedef streamoff                 off_type; 
  	typedef streampos                 pos_type;
  	typedef mbstate_t                 state_type;
#endif /* _RW_STD_IOSTREAM */

        static void assign (char_type& c1, const char_type& c2)   
         { c1 = c2; }

	static char_type         to_char_type(const int_type& c)
	 { return (char)c; }

  	static int_type          to_int_type(const char_type& c)
	 { return (unsigned char)c; }

	static bool              eq(const char_type& c1,const char_type& c2)
    	{ return (c1 == c2); }

        static bool lt (const char_type& c1, const char_type& c2) 
        { return c1 < c2; }

        static int compare (const char_type* s1, const char_type* s2, size_t n)
        { return memcmp(s1, s2, n); }

       static const char_type* 
       find (const char_type* s, int n, const char_type& a)
       { return (char_type*) memchr(s,a,n); }

  	static bool         eq_int_type(const int_type& c1,const int_type& c2)
	 { return (c1 == c2); }

  	static int_type          eof()
    	{ return EOF; }

  	static int_type          not_eof(const int_type& c)
	 {
           if ( c == EOF )
            return 0;
           else
            return c;
	 }
  			
  	static size_t            length(const char_type *s)
	 { return strlen(s); }

  	static char_type  *copy(char_type *dst,const char_type *src, size_t n)
    	{ return (char_type *) memcpy(dst, src, n); }


        static char_type * move (char_type* s1, const char_type* s2, size_t n)
        {
          #ifndef _RWSTD_NO_MEMMOVE
           return (char_type *) memmove(s1, s2, n);
          #else
           char_type * s = s1;
           if (s1 < s2)
            copy(s1, s2, n);
           else if (s1 > s2)
           {
            s1 += n;
            s2 += n;
            for(size_t i = 0; i < n; ++i) assign(*--s1, *--s2);
           }
          return s;
          #endif
        }

        static char_type*
        assign (char_type* s, size_t n, const char_type& a)
        {
          memset(s,a,n);
          return s;
        }

 };


/*
 *
 *     STRUCT CHAR_TRAITS SPECIALIZED FOR WCHAR_T
 *
 */


#ifndef _RWSTD_NO_WIDE_CHAR

_RWSTD_TEMPLATE
struct _RWSTDExport char_traits<wchar_t> 
{
  	typedef wchar_t                   char_type;

#ifndef _RWSTD_NO_WINT_TYPE
  	typedef wint_t                    int_type;
#else
  	typedef long                      int_type;
#endif

#ifndef WEOF
#define WEOF (-1)
#endif

#ifdef _RW_STD_IOSTREAM
        typedef wstreamoff                off_type;
  	typedef wstreampos                pos_type;
  	typedef mbstate_t                 state_type;
#endif /* _RW_STD_IOSTREAM */        

        static void assign (char_type& c1, const char_type& c2)   
        { c1 = c2;}

  	static char_type to_char_type(const int_type& c)
  	{ return c; }

	static int_type to_int_type(const char_type& c)
  	{ return c; }

	static bool     eq(const char_type& c1,const char_type& c2)
  	{ return (c1 == c2); }

        static bool lt (const char_type& c1, const char_type& c2) 
        { return c1 < c2;}

        static int compare (const char_type* s1, const char_type* s2, size_t n)
        {
         #ifndef _RWSTD_NO_WSTR
          return memcmp(s1, s2, n*sizeof(char_type));
         #else
          size_t i=0;

          while ( i < n )
           { 
             if ( !eq( s1[i], s2[i] ) )
              {
                 if ( lt( s1[i], s2[i]) )
                  return -1;
                 else
                  return 1;               
               }
             i++;
           }
         return 0;
         #endif
        }

  	static bool     eq_int_type(const int_type& c1,const int_type& c2)
  	{ return (c1 == c2); }

  	static int_type             eof()
  	{ return WEOF; }

  	static int_type             not_eof(const int_type& c)
  	{
    		if(c == WEOF)
      		   return 0;
    		else
      		   return c;
  	}

  	static size_t               length(const char_type *s)
  	{
	#ifndef _RWSTD_NO_WSTR   
    		return wcslen(s);
	#else
    	     size_t l = 0;
             while ( !eq(*s++, char_type(0) ) )  ++l;
             return l;
	#endif
  	}
 
        static const char_type* 
        find (const char_type* s, int n, const char_type& a)
        {
        #ifndef _RWSTD_NO_WSTR 
           return (char_type*) wmemchr(s,a,n);
      	#else
          while (n-- > 0 && *s != a) 
            ++s;
          return  n >= 0 ? s : 0;
      	#endif
        }

  	static char_type    *copy(char_type *dst,const char_type *src, size_t n)
  	{
      		return (char_type*) memcpy((char *)dst,(char *)src,
                                           n*sizeof(char_type));
  	}

        static char_type * move (char_type* s1, const char_type* s2, size_t n)
        {
          #ifndef _RWSTD_NO_MEMMOVE
           return (char_type *) memmove(s1, s2, n*sizeof(char_type));
          #else
           char_type * s = s1;
           if (s1 < s2)
            copy(s1, s2, n);
           else if (s1 > s2)
           {
            s1 += n;
            s2 += n;
            for(size_t i = 0; i < n; ++i) assign(*--s1, *--s2);
           }
          return s;
          #endif
        }


        static char_type* 
        assign (char_type* s, size_t n, const char_type& a)
        {
        #ifndef _RWSTD_NO_WSTR 
          wmemset(s,a,n);
          return s;
        #else
          char_type* tmp = s;
          while (n-- > 0) 
            *tmp++ = a;
          return s;
        #endif
        }

 };

#endif


#ifndef _RWSTD_NO_NAMESPACE
} namespace __rwstd {
using namespace std;
#endif
//
// Implementation traits class, rw_traits, provides specialized
// algorithms to speed up find operations on char and wchar_t strings
//

template <class charT, class traits>
struct _RWSTDExportTemplate rw_traits
{
  static const charT* find(const charT* s, const charT* v)
  { 
    size_t slen = traits::length(s);
    size_t vlen = traits::length(v);
    for (size_t xpos = 0; (xpos + vlen) <= slen ; xpos++)
    {
        bool found = true;
        for (size_t i = 0; i < vlen ; i++)
        {
            if (!traits::eq(s[xpos + i], v[i]))
            {
                found = false;
                break;
            }
        }
        if (found)
            return &s[xpos];
    }

    return NULL;
  }

  static const charT* rfind(const charT* s, charT v, size_t pos)
  {
    const charT* p = s + pos;
    while (p >= s)
    {
      if (traits::eq(*p,v))
        return p;
      p--;
    }
    return NULL;       
  }

  static size_t find_first_of(const charT* s, const charT* v)
  {
    const charT* p = s;
    for (; *p; p++)
    {
      for (const charT* q = v; *q; q++)
        if (traits::eq(*p, *q))
           return p-s;
    }
    return  p-s;
  }

  static size_t find_first_not_of(const charT* s, const charT* v)
  {
    const charT* p = s;
    for (; *p; p++)
    {
      for (const charT* q = v; *q; q++)
        if (!traits::eq(*p, *q))
           return p-s;
    }
    return  p-s;
  }
};

_RWSTD_TEMPLATE
struct _RWSTDExport rw_traits<char,char_traits<char> >
{
  static const char* find(const char* s, const char* v)
  { return strstr(s,v); }
  static const char* rfind(const char* s, char v, size_t pos)
  { 
    const char* p = s + pos;
    while (p >= s)
    {
      if (char_traits<char>::eq(*p,v))
        return p;
      p--; 
    }
    return NULL;       
  }
  static size_t find_first_of(const char* s, const char* v)
  { return strcspn(s,v); }
  static size_t find_first_not_of(const char* s, const char* v)
  { return strspn(s,v); }
};

#ifndef _RWSTD_NO_WSTR
_RWSTD_TEMPLATE
struct _RWSTDExport rw_traits<wchar_t,char_traits<wchar_t> >
{
  static const wchar_t* find(const wchar_t* s, const wchar_t* v)
  { return wcsstr(s,v); }
  static const wchar_t* rfind(const wchar_t* s, wchar_t v, size_t pos)
  { 
    const wchar_t* p = s + pos;
    while (p >= s)
    {
      if (char_traits<wchar_t>::eq(*p,v))
        return p;
      p--;
    }
    return NULL;       
  }
  static size_t find_first_of(const wchar_t* s, const wchar_t* v)
  { return wcscspn(s,v); }
  static size_t find_first_not_of(const wchar_t* s, const wchar_t* v)
  { return wcsspn(s,v); }
};
#endif // _RWSTD_NO_WSTR

#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#pragma option pop
#endif // __RW_TRAITS
